from setuptools import setup

setup(
    name='navalmartin_mir_db_utils',
    packages=['navalmartin_mir_db_utils'],
    version='0.0.1',
    description='Various utilities for working with AWS in Python 3.',
    author='Alexandros Giavaras',
    author_email='a.giavaras@gmail.com',
    url='https://github.com/Navalmartin/mir_persistence_layer_utils',
    download_url='https://github.com/pockerman/awsutils/awsutils-0.0.3.tar.gz',
    keywords=['mongodb', 'mongodb-utilities'],
    classifiers=[],
    scripts=[],
)
